#! /usr/local/bin/perl -w


# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see
# <http://www.gnu.org/licenses/>.



use Getopt::Std;

###################################################################################################
# Read the command line option
##################################################################################################
getopts('has:t:n', \%opts);

if((exists $opts{a} && exists $opts{s}) || (exists $opts{a} && exists $opts{t}) || (exists $opts{s} && exists $opts{t}) || exists $opts{h}) {
  &usage();
} # if


$all_context = 0;
$sentence_context = 0;
$token_context = 10;


if(exists $opts{a}) {
  $all_context = 1;
  $token_context = 0;
} # if

if(exists $opts{s}) {
  $sentence_context = $opts{s};
  $token_context = 0;
} # if

if(exists $opts{t}) {
  $token_context = $opts{t};
} # if


if(exists $opts{n}) {
  $filter_none_senses = 1;
} else {
  $filter_none_senses = 0;
} # if/else



###################################################################################################
# Start of script proper
###################################################################################################

%example_ids = ();
%contexts = ();
%ambig_token = ();
%none_examples = ();   

$example_counter = 0;
while(<>) {
  # Read through context, storing tokens and sentence breaks. 
  # Print back out in correct format based on defined input

  if(/^(Context\:.+)/) {
    $example_counter++;
    $example_ids{$example_counter} = $1;
    $context_counter = 0;
    
    if(/None\:None/) {
      $none_examples{$example_counter}++;
    } # if

  } elsif(/\#w[0-9]+\#/) {
    
    $context = $_;
    chomp($context);

 
    if($all_context == 1 || $sentence_context > 0) {
      $contexts{$example_counter}{$context_counter} = $context;
      if($context =~ /\#w[0-9]+\#1/) {
	$ambig_token{$example_counter} = $context_counter;
      } # if
      $context_counter++;
    } else {

      @tokens = split(/ +/, $context);
      foreach $token (@tokens) {
	$contexts{$example_counter}{$context_counter} = $token;
	if($token =~ /\#w[0-9]+\#1/) {
	  $ambig_token{$example_counter} = $context_counter;
	} # if
	$context_counter++;
	
      } # foreach 
   
    } # if/else
  
  } # if

} # while



# Print out reformatted data
foreach $example_counter (sort {$a <=> $b} keys %contexts) {
  if($filter_none_senses == 1 && exists $none_examples{$example_counter}) {
    next;
  } # if

  print "$example_ids{$example_counter}\n";
  
  # Decide what to print out
  if($all_context == 1) {
    $context_start = 0;
    $context_end = 100000;
  } else {
    if($sentence_context > 0) {
      $context_size = $sentence_context;
    } elsif($token_context > 0) {
      $context_size = $token_context;
    } # if/else
    $context_start = $ambig_token{$example_counter} - $context_size;
    $context_end = $ambig_token{$example_counter} + $context_size;
  } # if/else

  foreach $context_counter (sort {$a <=> $b} keys %{$contexts{$example_counter}}) {
    if($context_counter < $context_start || $context_counter > $context_end) {
      # Do nothing
    } else {
      print "$contexts{$example_counter}{$context_counter} ";
    } # foreach 
  } # foreach 
  print "\n\n";

} # foreach 



##################################################################################################
#  Subroutines
##################################################################################################

sub usage {
  
  die("$0 -a -s num -t num -n\n\t-a(ll):\tall context\n\t-s(entence):\tnumber of sentences in context\n\t-t(okens):\tnumber of tokens in context (default to 10)\n\t-n(one):\tfilter \"None\" senses\n\t-h(elp):\tprints these usage notes\n\n");

} # sub usage
