#! /usr/local/bin/perl -w

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see
# <http://www.gnu.org/licenses/>.

use Getopt::Std;


######################################################################################################
#   GLOBAL VARIABLES
######################################################################################################

# Location of NLM-WSD corpus
$nlm_wsd_corpus_dir =
"/share/nlp/projects/LexDis/data/nlm_WSD_corpus/Full_Test_Collection/Reviewed_Results/";

# MetaMap command 
my $metamap_home = "/share/nlp/projects/LexDis/tools/MetaMap2007/public_mm/bin/";
my $metamap_command = $metamap_home."metamap07 -cI";


######################################################################################################
#   COMMAND LINE OPTIONS
######################################################################################################

getopts('o:h', \%opts);

if(exists $opts{h}) { &usage(); } 


# Output directory
$out_dir = "./OUT";
if(exists $opts{o}) {
  $out_dir = $opts{o}."/";
} # if





######################################################################################################
#   SETUP SYSTEM
######################################################################################################

# Start the POS tagger and WSD servers

my $command = $metamap_home."/skrmedpostctl start";
system($command);
$command = $metamap_home."/wsdserverctl start";
system($command);


######################################################################################################
#   START OF SCRIPT PROPER
######################################################################################################

$command = "ls $nlm_wsd_corpus_dir/*/*_set |";
open(PIPE, $command) or die "Can't execute command as pipe: $command\n";
while(<PIPE>) {
  print;

  $file = $_;

  if($file =~ /\/([a-z\_]+)\_set/) {
    $ambig_term = $1;

    $output_file = $out_dir."/".$ambig_term;
    open(OUT, ">$output_file") or die "Can't open $output_file for writing\n";
  } # if

  open(FILE, $file) or die "Can't open $file for reading\n";
  while(<FILE>) {
    if(/^([0-9]+\|[0-9]+\.[abti]+\.[0-9]+\|(None|M[0-9]))/) {
      $counter = 0;
      print OUT "$ambig_term:";
      print OUT;
    } elsif(/\|[0-9]+\|[0-9]+\|([0-9]+)\|([0-9]+)\|/) {
      $ambig_start = $1;
      $ambig_end = $2;

      # Only print out the whole abstract
      unless($counter == 0) {

	# Break around ambiguous term
	$text =~ s/^\n+//;
	
	$pre_term = substr $text, 0, $ambig_start;
	$post_term = substr $text, ($ambig_end + 1), length $text;
	
	%doc = ();
	$doc{1} = $pre_term;
	$doc{2} = $ambig_term;
	$doc{3} = $post_term;

	foreach $doc_section (sort {$a <=> $b} keys %doc) {

	  $text = $doc{$doc_section};
	  if($doc_section == 2) {
	    print OUT "** Ambiguous term **\n";
	  } # if

	  $text =~ s/PMID\- [0-9]+//;
	  $text =~ s/TI +\- +//g;
	  $text =~ s/AB +\- +//g;
	  
	  $text =~ s/\"/\\\"/g;
	  
	  $command = "echo \"".$text."\" | ".$metamap_command." |";
	  open(METAMAP, $command) or die "Can't open pipe for reading: $command\n";
	  $print_flag = 0;
	  while(<METAMAP>) {
	    if(/^Processing [0-9]+\.tx\./) {
	      $print_flag++;
	    } # if
	    unless($print_flag == 0) { print OUT; } 
	  } # while
	  print OUT "\n";

	} # foreach 

      } # unless
	
      $text = "";
      $counter++;

    } else {
      $text .= $_;
    } # if/else
      
  } # while
  close(FILE);
  close(OUT);

} # while



######################################################################################################
#   CLEAN UP
######################################################################################################

# Stop the servers
$command = $metamap_home."/skrmedpostctl stop";
system($command);
$command = $metamap_home."/wsdserverctl stop";
system($command);



######################################################################################################
#   SUBROUTINES
######################################################################################################


sub usage {
  
  die("$0 -o outdir -h\n\t-o(utput directory, default to \"OUT/\")\n\t-h(elp, these usage notes)\n\n");

} # sub usage
